<?php
/*
phpdtaus - builder and parser for dtaus format

Copyright (C) 2009 Lukas Mueller

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, see <http://www.gnu.org/licenses/>.
*/

class Dtaus_Builder
{
 /**
  * @var string Type of DTAUS (L = Lastschrift, G = Ueberweisung)
  */
 protected $type = 'L';
 
 /**
  * @var string Subtype
  */  
 protected $detailedType; // = '05000';
 
 /**
  * @var string Execution date in format DDMMYY
  */
 protected $execDate; // date of execution
 
 /**
  * @var int Referencenumber
  */
 protected $reference;
 
 /**
  * @var string Name of sender
  */
 protected $senderName;
 
 /**
  * @var int BLZ
  */
 protected $senderBank;
 
 /**
  * @var string Accountid
  */
 protected $senderAccount;
 
 /**
  * @var array array with entries
  */
 protected $entries = array(); // array for entries
 
 /**
  * @var array array with checksums
  */
 protected $checksums = array('entries' => 0, 'accountsum' => 0, 'banksum' => 0, 'amountsum' => 0);
 
 /**
  * @var string Extenions will be added to every entry
  */
 protected $globalExtensions = array();
 
 
 /**
  * Constructor
  * @param string $sender_name Sendername
  * @param int  $sender_bank BLZ
  * @param int  $sender_account ID of account
  */
 public function __construct($sender_name, $sender_bank, $sender_account)
 {
  if(is_array($sender_name)){
   $this->senderName = $sender_name[0];
   
   for ($i = 1; $i < count($sender_name); $i++){
    $this->globalExtensions[] = array('type' => 3, 'text' => $sender_name[$i]);
   }
  }
  else{
   // cut sender_name into extensions if longer than 27
   $sender_name = $this->_convertText($sender_name);
   
   if (strlen($sender_name) > 27){
    $this->globalExtensions[] = array('type' => 3, 'text' => substr($sender_name, 27, 27));
    $sender_name = substr($sender_name, 0, 27);
   }
   $this->senderName = $sender_name;
  }
  
  $this->senderBank = (int)$sender_bank;
  $this->senderAccount = (int)$sender_account;
 }
 
 /**
  * Sets type of DTAUS
  * @param string $type L = Lastschrift; G = Gutschrift
  */
 public function setType($type, $detailedType = '')
 {
  $this->type = $type;
  $this->detailedType = $detailedType;
 }
 
 /**
  * Sets execution date of DTAUS
  * @param string $execTime DDMMYY
  */
 public function setExecDate($execTime)
 {
  $this->execTime = $execTime;
 }
 
 /**
  * Sets reference number
  * @param integer $reference Reference Number
  */
 public function setReference($reference)
 {
  $this->reference = (int)$reference;
 }
 
 /**
  * Prefix string with zeros
  * @param int  $count  length of expected string
  * @param string $text  string with text used (optional)
  * @return string    new string, prefixed with zeros
  */
 protected function _pre0($count, $text = ''){
  return str_pad(substr($text, 0, $count), $count, '0', STR_PAD_LEFT);
 }
 
 /**
  * Suffix string with spaces
  * @param int  $count  length of expected string
  * @param string $text  string with text used (optional)
  * @return string    new string, suffixed with spaces
  */
 protected function _sufE($count, $text = ''){
  return str_pad(substr($this->_convertText($text), 0, $count), $count, ' ', STR_PAD_RIGHT);
 }

 /**
  * Convert text to upper, replace umlauts
  * @param string $text  string with text used
  * @return string    new converted string
  */
 protected function _convertText($text){
  return strtoupper(str_replace(array('ä', 'ö', 'ü', 'Ä', 'Ö', 'Ü', 'ß'), array('AE', 'OE', 'UE', 'AE', 'OE', 'UE', 'SS'), $text));
 }
 
 /**
  * sort array with typeid
  * @param array $a1 first array
  * @param array $a2 first array
  * @return string  0, -1 or 1
  */
 protected function _sortExtensions($a1, $a2)
 {
   echo $a1['type'] . "_" . $a2['type'] . "\n\n";
  if ($a1['type'] == $a2['type'])
   return 0;
  
  return ($a1['type'] < $a2['type']) ? -1 : 1;
 }
 
 /**
  * add new entry
  * @param string|array $reciver_name  array with name, if is string, it will be cut every 27 signs
  * @param int    $bank   BLZ
  * @param int    $account  accountid
  * @param float   $amount    amount, example: 12.34
  * @param string|array $reference  array with reference, if is string, it will be cut every 27 signs (optional)
  * @param int    $customerid  ID for customer (optional)
  */
 public function add($reciver_name, $reciver_bank, $reciver_account, $reciver_amount, $reference = array(), $customerid = 0)
 {
  $extensions = array();
  
  if (!is_array($reference))
   $reference = str_split($reference, 27);
  if (!is_array($reciver_name))
   $reciver_name = str_split($reciver_name, 27);
   
  array_map(array($this, '_convertText'), $reference, $reciver_name);
  
  
  for ($i = 1; $i < count($reference); $i++)
   $extensions[] = array('type' => 2, 'text' => $reference[$i]);
  for ($i = 1; $i < count($reciver_name); $i++)
   $extensions[] = array('type' => 1, 'text' => $reciver_name[$i]);
   
  $reciver_amount = $reciver_amount * 100;
  $this->entries[] = array(
    'name' => $reciver_name[0],
    'bank' => $reciver_bank,
    'account' => $reciver_account,
    'amount' => $reciver_amount,
    'reference' => $reference[0],
    'extensions' => array_merge($extensions, $this->globalExtensions),
    'customerid' => (int)$customerid
   );
   
  $this->checksums['entries']++;
  $this->checksums['accountsum'] += $reciver_account;
  $this->checksums['banksum']  += $reciver_bank;
  $this->checksums['amountsum'] += $reciver_amount;
 }
 
 /**
  * Get string with data
  * @return string  string with created DTAUS
  */
  
 public function create($feldtrenner=''){
  if (empty($this->detailedType))
   $this->detailedType = ($this->type == 'L'? '05000' : '51000');
  
  // PART A = header
  $out = '0128'.$feldtrenner; // length
  $out .= 'A'.$feldtrenner; // type
  $out .= $this->type.'K'.$feldtrenner; // type: LK = Lastschrift, GK = Überweisung
  $out .= $this->_pre0(8, $this->senderBank).$feldtrenner; // BLZ
  $out .= $this->_pre0(8).$feldtrenner; // just zeros
  $out .= $this->_sufE(27, $this->senderName).$feldtrenner; // sender
  $out .= date('dmy').$feldtrenner; // creation date
  $out .= $this->_sufE(4).$feldtrenner; // empty string
  $out .= $this->_pre0(10, $this->senderAccount).$feldtrenner; // account number
  $out .= $this->_pre0(10, $this->reference).$feldtrenner; // reference
  $out .= $this->_sufE(15) . $this->_sufE(8, $this->execDate) . $this->_sufE(24).$feldtrenner; // exec date
  $out .= '1'.$feldtrenner.$feldtrenner; // currency = Euro
  
  foreach($this->entries as $entry){
   // PART C = data
   $out .= $this->_pre0(4, (187 + (count($entry['extensions']) * 29))).$feldtrenner; // length + (textcount * 29)
   $out .= 'C'.$feldtrenner; // type
   $out .= $this->_pre0(8, $this->senderBank).$feldtrenner; // blz target
   $out .= $this->_pre0(8, $entry['bank']).$feldtrenner; // blz source
   $out .= $this->_pre0(10, $entry['account']).$feldtrenner; // account source
   $out .= $this->_pre0(13, $entry['customerid']).$feldtrenner; // customer id
   $out .= $this->detailedType.$feldtrenner; // Lastschrift = 05000; Gutschrift = 51000
   $out .= ' '.$feldtrenner;
   $out .= '00000000000'.$feldtrenner; // amount in DM
   $out .= $this->_pre0(8, $entry['bank']).$feldtrenner; // blz target
   $out .= $this->_pre0(10, $entry['account']).$feldtrenner; // account target
   $out .= $this->_pre0(11, $entry['amount']).$feldtrenner; // amount   
   $out .= '   '.$feldtrenner;
   $out .= $this->_sufE(27, $entry['name']).$feldtrenner; // name source
   $out .= '        '.$feldtrenner;
   $out .= $this->_sufE(27, $this->senderName).$feldtrenner; // name target
   $out .= $this->_sufE(27, $entry['reference']).$feldtrenner; // text 1
   $out .= '1'.$feldtrenner; // currency = EUR
   $out .= '  '.$feldtrenner;
   $out .= $this->_pre0(2, count($entry['extensions'])).$feldtrenner;
   
   // usort($entry['extensions'], array($this, '_sortExtensions')); //lm: currently disabled
   
   // and now this damn, ugly, stupid extensions
   $i = 0; // counter for parts
   $extensionCount = count($entry['extensions']);
   
   while($i != 2){
   
    if (!isset($entry['extensions'][$i]))
     $out .= $this->_pre0(29);
    else
    {
     $out .= $this->_pre0(2, $entry['extensions'][$i]['type']);
     $out .= $this->_sufE(27, $entry['extensions'][$i]['text']);
    }
    $i++;
   }
   $out .= $this->_sufE(11); 

   // now 4 blocks; 4th block just 1 part, needs extension
   $blockCount = 0;
   while($i < $extensionCount && $blockCount < 4)
   {
    $blockCount++;
                for ($x = 0; ($x < 4 && $blockCount < 4) || $x < 1; $x++)
                {
                    if ($i < $extensionCount)
                    {
      $out .= $this->_pre0(2, $entry['extensions'][$i]['type']);
      $out .= $this->_sufE(27, $entry['extensions'][$i]['text']);
                        $i++;
                    }
                    else
      $out .= $this->_sufE(29);
    }
                if ($blockCount == 4) // 4th block has 1 extension, must be 128 byte
     $out .= $this->_sufE(29 * 3);
    
    $out .= $this->_sufE(12);
   }
  }
  
  // PART E
  $out .= '0128'; // length
  $out .= 'E'; // type
  $out .= '     '; // "     " Erm, yeah. Another unknown thing
  $out .= $this->_pre0(7, $this->checksums['entries']);
  $out .= '0000000000000'; // sum in DM; no more required
  $out .= $this->_pre0(17, $this->checksums['accountsum']); // sum of accounts
  $out .= $this->_pre0(17, $this->checksums['banksum']); // sum of blz
  $out .= $this->_pre0(13, $this->checksums['amountsum']); // sum of amounts
  $out .= $this->_sufE(51);

  return $out;
 }
}
?>